@@ -156,14 +156,15 @@ class IpTree(object):
 
         unallocated_nodes = self.unallocated_nodes
         key_func  = lambda x: x.get(group_attr)
-        if all(isinstance(item, autonetkit.anm.overlay_interface) for item in unallocated_nodes):
+        if all((isinstance(item, autonetkit.anm.overlay_interface) and item.is_loopback)
+                for item in unallocated_nodes):
             key_func  = lambda x: x.node.get(group_attr) # interface, map key function to be the interface's node
 
         unallocated_nodes = sorted(unallocated_nodes, key = key_func)
         groupings = itertools.groupby(unallocated_nodes, key = key_func)
         for attr_value, items in groupings:
 # make subtree for each attr
-            items = list(items)
+            items = sorted(list(items))
             subgraph = nx.DiGraph()
 
             if all(isinstance(item, autonetkit.anm.overlay_interface) for item in items):
@@ -220,17 +221,17 @@ class IpTree(object):
             for level, nodes in nodes_by_level.items():
                 level_counts[level] = len(nodes)
 
-            min_level = min(level_counts)
-            # need to update level counts if added extra parent nodes,
-            # in order to ensure root node isn't a cd
-            if level_counts[min_level] == 1:
-                # one node at this level -> added one parent at next level (node is a cd)
-                level_counts[min_level - 1] = 1
-            elif level_counts[min_level] == 2:
-                # two nodes at this level -> added a grandparent (nodes are cds)
-                level_counts[min_level - 2] = 1
-
             self.add_parent_nodes(subgraph, level_counts)
+# test if min_level node is bound, if so then add a parent, so root for AS isn't a cd
+            min_level = min(level_counts)
+            min_level_nodes = [n for n in subgraph if subgraph.node[n]['prefixlen'] == min_level]
+            # test if bound
+            if len(min_level_nodes) == 2:
+                subgraph.add_node(self.next_node_id, {'prefixlen': min_level - 2})
+                subgraph.add_node(self.next_node_id, {'prefixlen': min_level - 2})
+                subgraph.add_node(self.next_node_id, {'prefixlen': min_level - 1})
+            if len(min_level_nodes) == 1:
+                subgraph.add_node(self.next_node_id, {'prefixlen': min_level - 1})
 
             # rebuild with parent nodes
             nodes_by_level = defaultdict(list)
@@ -293,7 +294,7 @@ class IpTree(object):
                 #TODO: sort these
                 child_id = self.next_node_id
                 cd_id = cd.node
-                global_graph.add_node(child_id, prefixlen = 32, host = edge)
+                global_graph.add_node(child_id, prefixlen = 32, host = edge.dst_int)
                 global_graph.add_edge(cd_id, child_id) # cd -> neigh (cd is parent)
 
 #TODO: make allocate seperate step
@@ -318,17 +319,45 @@ class IpTree(object):
                 iterhosts = node.subnet.iter_hosts() # ensures start at .1 rather than .0
                 sub_children = node.children()
                 for sub_child in sub_children:
-                    sub_child.subnet = iterhosts.next()
+                    #TODO: tidy up this allocation to always record the subnet
+                    if sub_child.is_interface() and sub_child.host.is_loopback:
+                        if sub_child.host.is_loopback_zero:   
+                            # loopback zero, just store the ip address
+                            sub_child.ip_address = iterhosts.next()
+                        else:
+                            # secondary loopback
+                            sub_child.ip_address = iterhosts.next()
+                            sub_child.subnet = node.subnet
+                    elif sub_child.is_interface() and sub_child.host.is_physical:
+                        # physical interface
+                        sub_child.ip_address = iterhosts.next()
+                        sub_child.subnet = node.subnet
+                    else:
+                        sub_child.subnet = iterhosts.next()
 
                 return
 
             for child in children:
+                # traverse the tree
                 if child.is_collision_domain():
-                    child.subnet = subnet.next()
+                    subnet = subnet.next()
+                    child.subnet = subnet
                     iterhosts = child.subnet.iter_hosts() # ensures start at .1 rather than .0
                     sub_children = child.children()
                     for sub_child in sub_children:
-                        sub_child.subnet = iterhosts.next()
+                        if sub_child.is_interface():
+                            interface = sub_child.host
+                            if interface.is_physical:
+                                # physical interface
+                                sub_child.ip_address = iterhosts.next()
+                                sub_child.subnet = subnet 
+                            elif interface.is_loopback and not interface.is_loopback_zero:
+                                # secondary loopback interface
+                                sub_child.ip_address = iterhosts.next()
+                                sub_child.subnet = subnet 
+                        else:
+                            sub_child.subnet = iterhosts.next()
+
                         log.debug( "Allocate sub_child to %s %s" % ( sub_child, sub_child.subnet))
                 elif child.is_host():
                     child.subnet = subnet.next()
@@ -337,7 +366,12 @@ class IpTree(object):
                     iterhosts = child.subnet.iter_hosts() # ensures start at .1 rather than .0
                     sub_children = child.children()
                     for sub_child in sub_children:
-                        sub_child.subnet = iterhosts.next()
+                        if sub_child.is_interface() and not sub_child.host.is_loopback_zero:
+                           # secondary loopback
+                           sub_child.ip_address = iterhosts.next()
+                           sub_child.subnet = child.subnet 
+                        else:
+                            sub_child.subnet = iterhosts.next()
                 else:
                     child.subnet = subnet.next()
                     allocate(child) # continue down the tree
@@ -396,10 +430,6 @@ class IpTree(object):
     
     def assign(self):
 # assigns allocated addresses back to hosts
-        edges = [n for n in self if n.is_host() and n.host.src]
-        for edge in edges:
-            edge.host.ip_address = edge.subnet
-    
         # don't look at host nodes now - use loopback_groups
         host_tree_nodes = [n for n in self if n.is_host() and n.host.is_l3device]
         #for host_tree_node in host_tree_nodes:
@@ -413,7 +443,17 @@ class IpTree(object):
 
         interfaces = [n for n in self if n.is_interface()]
         for n in interfaces:
-            n.host.loopback = n.subnet
+            interface = n.host
+            if interface.is_loopback and interface.is_loopback_zero:
+                # primary loopback
+                interface.loopback = n.ip_address
+            elif interface.is_loopback and not interface.is_loopback_zero:
+                # secondary loopback
+                interface.loopback = n.ip_address
+                interface.subnet = n.subnet
+            elif interface.is_physical:
+                interface.ip_address = n.ip_address
+                interface.subnet = n.subnet
 
 def assign_asn_to_interasn_cds(g_ip, address_block = None):
     G_phy = g_ip.overlay("phy")
@@ -446,7 +486,6 @@ def allocate_infra(g_ip, address_block = None):
 
 #TODO: apply directly here
 
-
 def allocate_loopbacks(g_ip, address_block = None):
     if not address_block:
         address_block = netaddr.IPNetwork("192.168.0.0/22")
@@ -461,17 +500,24 @@ def allocate_loopbacks(g_ip, address_block = None):
 def allocate_vrf_loopbacks(g_ip, address_block = None):
     if not address_block:
         address_block = netaddr.IPNetwork("172.16.0.0/24")
+
+    secondary_loopbacks = [i for n in g_ip.nodes()
+        for i in n.loopback_interfaces
+        if not i.is_loopback_zero
+        ]
+
+    if not len(secondary_loopbacks):
+            return # nothing to set
     log.info("Allocating v4 Secondary Host loopback IPs")
     ip_tree = IpTree(address_block)
-    secondary_loopbacks = [i for n in g_ip.nodes()
-            for i in n.loopback_interfaces
-            if not i.is_loopback_zero
-            ]
+
 
     vrf_loopbacks = [i for i in secondary_loopbacks
             if i['vrf'].vrf_name]
 
     ip_tree.add_nodes(vrf_loopbacks)
+
+
     ip_tree.build()
     #secondary_loopback_tree = ip_tree.json()
     ip_tree.assign()
